#!/bin/sh
# Initial Asahi Linux stuff

PREREQ=""
prereqs()
{
     echo "$PREREQ"
}

case $1 in
prereqs)
     prereqs
     exit 0
     ;;
esac

. /scripts/functions
# Begin real processing below this line


mount_sys_esp() {
    set -e
    mountpoint="$1"

    mkdir -p "$mountpoint"
    while grep -q "$mountpoint" /proc/mounts; do
        umount "$mountpoint"
    done

    esp_uuid="$(cat /proc/device-tree/chosen/asahi,efi-system-partition 2>/dev/null | sed 's/\x00//')"
    if [ -e /boot/efi/.builder ] || [ -e /boot/.builder ] || [ -z "$esp_uuid" ]; then
        if [ -e "/boot/efi/m1n1" ]; then
            bootmnt="/boot/efi"
        elif [ -e "/boot/m1n1" ]; then
            bootmnt="/boot"
        else
            log_warning_msg "ESP not found and cannot determine ESP PARTUUID."
            log_warning_msg "Make sure that your m1n1 has the right asahi,efi-system-partition configuration,"
            log_warning_msg "or that your ESP is mounted at /boot/efi or /boot."
            return 1
        fi
        mount --bind "$bootmnt" "$mountpoint"
        log_warning_msg "System ESP not identified in device tree, using $bootmnt"
    else
        mount "PARTUUID=$esp_uuid" "$mountpoint"
    fi
    dev="$(grep "$mountpoint" /proc/mounts | cut -d" " -f1)"
    log_success_msg "Mounted System ESP $dev at $mountpoint"
}



if [ -e /vendorfw ]; then
    log_warning_msg ":: Asahi: Vendor firmware was loaded by the bootloader"
    return 0
fi

if [ ! -e /proc/device-tree/chosen/asahi,efi-system-partition ]; then
    log_warning_msg ":: Asahi: Missing asahi,efi-system-partition variable, firmware will not be loaded!"
    return 1
fi

log_begin_msg ":: Asahi: Triggering early load of NVMe modules..."
modprobe apple-mailbox
modprobe nvme-apple
log_end_msg

for i in $(seq 0 50); do
    [ -e /sys/bus/platform/drivers/nvme-apple/*.nvme/nvme/nvme*/nvme*n1/ ] && break
    sleep 0.1
done

if [ ! -e /sys/bus/platform/drivers/nvme-apple/*.nvme/nvme/nvme*/nvme*n1/ ]; then
    log_failure_msg "Timed out waiting for NVMe device"
    return 1
fi

# If the above exists, hopefully the /dev device exists and this will work

log_begin_msg ":: Asahi: Unpacking vendor firmware into initramfs..."

VENDORFW="/run/.system-efi/vendorfw/"

(
    mount_sys_esp /run/.system-efi
)

if [ ! -e "$VENDORFW/firmware.cpio" ]; then
    log_warning_msg ":: Asahi: Vendor firmware not available in ESP!"
    umount /run/.system-efi
    return 1
fi

( cd /; cpio -i < "$VENDORFW/firmware.cpio" )
umount /run/.system-efi

log_end_msg


exit 0
